from __future__ import unicode_literals

import shutil
import tempfile
from os.path import join, exists

import mock

from binstar_client.scripts.cli import main
from binstar_client.tests.fixture import CLITestCase


class Test(CLITestCase):
    def test_write_env(self):
        tmpdir = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, tmpdir)

        with mock.patch('binstar_client.commands.config.USER_CONFIG', join(tmpdir, 'config.yaml')), \
             mock.patch('binstar_client.commands.config.SEARCH_PATH', [tmpdir]):
            main(['config', '--set', 'url', 'http://localhost:5000'], False)

            self.assertTrue(exists(join(tmpdir, 'config.yaml')))

            with open(join(tmpdir, 'config.yaml')) as f:
                config_output = f.read()
            expected_config_output = 'url: http://localhost:5000\n'
            self.assertEqual(config_output, expected_config_output)

            main(['config', '--show-sources'], False)
            expected_show_sources_output = '==> {config} <==\nurl: http://localhost:5000\n\n'.format(
                config=join(tmpdir, 'config.yaml'))

            self.assertIn(expected_show_sources_output, self.stream.getvalue())



import unittest

from binstar_client.errors import BinstarError
from binstar_client.utils.notebook import notebook_url, parse, has_environment
from binstar_client.utils.test.utils import data_dir


class ParseTestCase(unittest.TestCase):
    def test_parse(self):
        self.assertEqual(parse("user/notebook-ipynb")[0], 'user')
        self.assertEqual(parse("user/notebook-ipynb")[1], 'notebook-ipynb')

        self.assertIsNone(parse("notebook")[0])
        self.assertEqual(parse("notebook")[1], 'notebook')


class NotebookURLTestCase(unittest.TestCase):
    def test_anaconda_org_installation(self):
        upload_info = {'url': 'http://anaconda.org/darth/deathstart-ipynb'}
        url = 'http://notebooks.anaconda.org/darth/deathstart-ipynb'
        self.assertEqual(notebook_url(upload_info), url)

    def test_anaconda_server_installation(self):
        upload_info = {'url': 'http://custom/darth/deathstart-ipynb'}
        url = 'http://custom/notebooks/darth/deathstart-ipynb'
        self.assertEqual(notebook_url(upload_info), url)


class HasEnvironmentTestCase(unittest.TestCase):
    def test_has_no_environment(self):
        self.assertFalse(has_environment(data_dir('notebook.ipynb')))

    def test_has_environment(self):
        self.assertTrue(has_environment(data_dir('notebook_with_env.ipynb')))

    def test_no_file(self):
        self.assertFalse(has_environment("no-file"))


if __name__ == '__main__':
    unittest.main()



import os
import re
import time
from os.path import basename
from binstar_client import errors
from .inflection import parameterize
from .data_uri import data_uri_from

VALID_FORMATS = ['ipynb', 'csv', 'yml', 'yaml', 'json', 'md', 'rst', 'txt']


class Uploader(object):
    """
    * Find or create a package (project)
    * Find or create release (version)
    * List files from project
    * Upload new files to project
    """
    _package = None
    _release = None
    _project = None

    def __init__(self, aserver_api, filepath, **kwargs):
        self.aserver_api = aserver_api
        self.filepath = filepath
        self._username = kwargs.get('user', None)
        self._version = kwargs.get('version', None)
        self._summary = kwargs.get('summary', None)
        self._thumbnail = kwargs.get('thumbnail', None)
        if 'name' in kwargs and kwargs['name'] is not None:
            self._project = parameterize(kwargs['name'])

    def upload(self, force=False):
        """
        Uploads a notebook
        :param force: True/False
        :returns {}
        """
        self.package and self.release
        try:
            return self.aserver_api.upload(self.username, self.project, self.version,
                                       basename(self.filepath), open(self.filepath, 'rb'),
                                       self.filepath.split('.')[-1])
        except errors.Conflict:
            if force:
                self.remove()
                return self.upload()
            else:
                msg = "Conflict: {} already exist in {}/{}".format(self.filepath,
                                                                   self.project,
                                                                   self.version)
                raise errors.BinstarError(msg)

    def remove(self):
        return self.aserver_api.remove_dist(
            self, self.username, self.project, self.version, basename=self.notebook,
        )

    @property
    def notebook_attrs(self):
        if self._thumbnail is not None:
            return {'thumbnail': data_uri_from(self._thumbnail)}
        else:
            return {}

    @property
    def project(self):
        if self._project is None:
            return re.sub('\-ipynb$', '', parameterize(os.path.basename(self.filepath)))
        else:
            return self._project

    @property
    def username(self):
        if self._username is None:
            self._username = self.aserver_api.user()['login']
        return self._username

    @property
    def version(self):
        if self._version is None:
            self._version = time.strftime('%Y.%m.%d.%H%M')
        return self._version

    @property
    def summary(self):
        if self._summary is None:
            self._summary = "IPython notebook"
        return self._summary

    @property
    def package(self):
        if self._package is None:
            try:
                self._package = self.aserver_api.package(self.username, self.project)
            except errors.NotFound:
                self._package = self.aserver_api.add_package(self.username, self.project,
                                                         summary=self.summary,
                                                         attrs=self.notebook_attrs)
        return self._package

    @property
    def release(self):
        if self._release is None:
            try:
                self._release = self.aserver_api.release(self.username, self.project, self.version)
            except errors.NotFound:
                self._release = self.aserver_api.add_release(self.username, self.project,
                                                         self.version, None, None, None)
        return self._release

    @property
    def files(self):
        return self.package['files']



"""
[Deprecation warning]
`anaconda notebook` is going to be deprecated
use `anaconda upload/download` instead
"""

from __future__ import unicode_literals
import argparse
import logging
from binstar_client import errors
from binstar_client.utils import get_server_api
from binstar_client.utils.notebook import Uploader, Downloader, parse, notebook_url, has_environment

logger = logging.getLogger("binstar.notebook")


def add_parser(subparsers):
    description = 'Interact with notebooks in your Anaconda repository'
    parser = subparsers.add_parser('notebook',
                                   formatter_class=argparse.RawDescriptionHelpFormatter,
                                   help=description,
                                   description=description,
                                   epilog=__doc__)

    nb_subparsers = parser.add_subparsers()
    add_upload_parser(nb_subparsers)
    add_download_parser(nb_subparsers)


def add_upload_parser(subparsers):
    description = "Upload a notebook to your Anaconda repository"
    epilog = """
    [Deprecation warning]
    `anaconda notebook` is going to be deprecated
    use `anaconda upload` instead
    """
    parser = subparsers.add_parser('upload',
                                   formatter_class=argparse.RawDescriptionHelpFormatter,
                                   help=description,
                                   description=description,
                                   epilog=epilog)

    mgroup = parser.add_argument_group('metadata options')
    mgroup.add_argument('-n', '--name', help='Notebook\'s name (will be parameterized)')
    mgroup.add_argument('-v', '--version', help='Notebook\'s version')
    mgroup.add_argument('-s', '--summary', help='Set the summary of the notebook')
    mgroup.add_argument('-t', '--thumbnail', help='Notebook\'s thumbnail image')

    parser.add_argument(
        '-u', '--user',
        help='User account, defaults to the current user'
    )

    parser.add_argument(
        '--force',
        help="Force a notebook upload regardless of errors",
        action='store_true'
    )

    parser.add_argument(
        'notebook',
        help='Notebook to upload',
        action='store'
    )

    parser.set_defaults(main=upload)


def add_download_parser(subparsers):
    description = "Download notebooks from your Anaconda repository"
    epilog = """
    [Deprecation warning]
    `anaconda notebook` is going to be deprecated
    use `anaconda download` instead
    """
    parser = subparsers.add_parser('download',
                                   formatter_class=argparse.RawDescriptionHelpFormatter,
                                   help=description,
                                   description=description,
                                   epilog=epilog)

    parser.add_argument(
        'handle',
        help="user/notebook",
        action='store'
    )

    parser.add_argument(
        '-f', '--force',
        help='Overwrite',
        action='store_true'
    )

    parser.add_argument(
        '-o', '--output',
        help='Download as',
        default='.'
    )

    parser.set_defaults(main=download)


def upload(args):
    aserver_api = get_server_api(args.token, args.site)

    uploader = Uploader(aserver_api, args.notebook, user=args.user, summary=args.summary,
                        version=args.version, thumbnail=args.thumbnail, name=args.name)

    try:
        upload_info = uploader.upload(force=args.force)
        logger.warning("`anaconda notebook` is going to be deprecated")
        logger.warning("use `anaconda upload` instead.")
        logger.info("{} has been uploaded.".format(args.notebook))
        logger.info("You can visit your notebook at {}".format(notebook_url(upload_info)))
    except (errors.BinstarError, IOError) as e:
        logger.error(str(e))


def download(args):
    aserver_api = get_server_api(token=args.token, site=args.site)

    username, notebook = parse(args.handle)
    username = username or aserver_api.user()['login']
    downloader = Downloader(aserver_api, username, notebook)
    try:
        download_info = downloader(output=args.output, force=args.force)
        logger.warning("`anaconda notebook` is going to be deprecated")
        logger.warning("use `anaconda download` instead.")
        logger.info("{} has been downloaded as {}.".format(args.handle, download_info[0]))
        if has_environment(download_info[0]):
            logger.info("{} has an environment embedded.".format(download_info[0]))
            logger.info("Run:")
            logger.info("    conda env create {}".format(download_info[0]))
            logger.info("To install the environment in your system")
    except (errors.DestionationPathExists, errors.NotFound, OSError) as err:
        logger.info(err.msg)



"""
Update public attributes of the package or the attributes of the package release
"""
import json
from binstar_client.utils import parse_specs

import argparse
import logging
import os

from binstar_client import errors
from binstar_client.utils import get_server_api
from binstar_client.utils.detect import detect_package_type, get_attrs

logger = logging.getLogger('binstar.update')


def get_attributes(package, package_type, args):
    if package.endswith('.json'):
        with open(package, 'r') as f:
            attrs = json.load(f)
            return attrs, attrs

    package_attrs, release_attrs, file_attrs = get_attrs(package_type, package, parser_args=args)
    return package_attrs, release_attrs


def main(args):
    anaconda_api = get_server_api(args.token, args.site)
    anaconda_api.check_server()

    package_type = detect_package_type(args.source)
    try:
        package_attrs, release_attrs = get_attributes(args.source, package_type, args)
    except Exception:
        message = 'Trouble reading metadata from {}. Is this a valid source file: {} ?'.format(
            args.source, package_type.label())
        logger.error(message)
        raise errors.BinstarError(message)

    attrs = package_attrs if not args.release else release_attrs
    attrs = attrs.get('public_attrs', attrs)

    if args.release:
        anaconda_api.update_release(
            args.spec.user,
            args.spec.package,
            args.spec.version,
            attrs,
        )
    else:
        anaconda_api.update_package(
            args.spec.user,
            args.spec.package,
            attrs,
        )
    logger.info("Package `{}` has been updated!".format(args.spec))


def file_type(path):
    if os.path.isfile(path):
        return path

    raise argparse.ArgumentTypeError("{} is not a valid path".format(path))


def add_parser(subparsers):
    parser = subparsers.add_parser(
        'update',
        usage="\n\tanaconda update user/package[/version] CONDA_PACKAGE_1.bz2"
              "\n\tanaconda update user/package[/version] metadata.json",
        description=__doc__)

    package_help = ('Path to the file that consists of metadata that will be updated in the destination package. '
                    'It may be a valid package file or `.json` file with described attributes to update')

    parser.add_argument('spec', help='Package name written as `user/package[/version]`', type=parse_specs)
    parser.add_argument('source', help=package_help, type=file_type)
    release_group = parser.add_argument_group(title='Update release')
    release_group.add_argument(
        '--release',
        help='Use `source` file to update public attributes of the release specified in `spec`',
        action='store_true'
    )

    parser.set_defaults(main=main)



'''
Created on Sep 23, 2013

@author: sean
'''
from unittest.runner import TextTestRunner, TextTestResult
from unittest.signals import registerResult
import time
import sys

WARNING = '\033[33m'
OKBLUE = '\033[34m'
OKGREEN = '\033[32m'
FAIL = '\033[31m'
ENDC = '\033[0m'
BOLD = "\033[1m"

def green(text):
    return BOLD + OKGREEN + text + ENDC

def red(text):
    return BOLD + FAIL + text + ENDC

def orange(text):
    return WARNING + text + ENDC

def blue(text):
    return OKBLUE + text + ENDC

class ColorTextTestResult(TextTestResult):
    def addSuccess(self, test):
        super(TextTestResult, self).addSuccess(test)
        if self.showAll:
            self.stream.writeln(green("ok"))
        elif self.dots:
            self.stream.write('.')
            self.stream.flush()

    def addError(self, test, err):
        super(TextTestResult, self).addError(test, err)
        if self.showAll:
            self.stream.writeln(red("ERROR"))
        elif self.dots:
            self.stream.write(red('E'))
            self.stream.flush()

    def addFailure(self, test, err):
        super(TextTestResult, self).addFailure(test, err)
        if self.showAll:
            self.stream.writeln(red("FAIL"))
        elif self.dots:
            self.stream.write(red('F'))
            self.stream.flush()

    def addSkip(self, test, reason):
        super(TextTestResult, self).addSkip(test, reason)
        if self.showAll:
            self.stream.writeln(blue("skipped {0!r}".format(reason)))
        elif self.dots:
            self.stream.write(blue("s"))
            self.stream.flush()

    def addExpectedFailure(self, test, err):
        super(TextTestResult, self).addExpectedFailure(test, err)
        if self.showAll:
            self.stream.writeln(blue("expected failure"))
        elif self.dots:
            self.stream.write(blue("x"))
            self.stream.flush()

    def addUnexpectedSuccess(self, test):
        super(TextTestResult, self).addUnexpectedSuccess(test)
        if self.showAll:
            self.stream.writeln(blue("unexpected success"))
        elif self.dots:
            self.stream.write(blue("u"))
            self.stream.flush()

    def printErrors(self):
        if self.dots or self.showAll:
            self.stream.writeln()
        self.printErrorList(red('ERROR'), self.errors)
        self.printErrorList(red('FAIL'), self.failures)

class ColorTextTestRunner(TextTestRunner):

    def __init__(self, stream=sys.stderr, descriptions=True, verbosity=1,
        failfast=False, buffer=False, resultclass=ColorTextTestResult):
        
        TextTestRunner.__init__(self, stream=stream, descriptions=descriptions,
                                verbosity=verbosity, failfast=failfast, buffer=buffer,
                                resultclass=resultclass)
    def run(self, test):
        "Run the given test case or test suite."
        result = self._makeResult()
        registerResult(result)
        result.failfast = self.failfast
        result.buffer = self.buffer
        startTime = time.time()
        startTestRun = getattr(result, 'startTestRun', None)
        if startTestRun is not None:
            startTestRun()
        try:
            test(result)
        finally:
            stopTestRun = getattr(result, 'stopTestRun', None)
            if stopTestRun is not None:
                stopTestRun()
        stopTime = time.time()
        timeTaken = stopTime - startTime
        result.printErrors()
        if hasattr(result, 'separator2'):
            self.stream.writeln(result.separator2)
        run = result.testsRun
        self.stream.writeln("Ran %d test%s in %.3fs" % 
                            (run, run != 1 and "s" or "", timeTaken))
        self.stream.writeln()


        return result

    def write_end(self, result):

        expectedFails = unexpectedSuccesses = skipped = 0
        try:
            results = map(len, (result.expectedFailures,
                                result.unexpectedSuccesses,
                                result.skipped))
        except AttributeError:
            pass
        else:
            expectedFails, unexpectedSuccesses, skipped = results
            
        infos = []
        if not result.wasSuccessful():
            self.stream.write('Tests: ' + red("FAILED"))
            failed, errored = map(len, (result.failures, result.errors))
            if failed:
                infos.append("failures=%d" % failed)
            if errored:
                infos.append("errors=%d" % errored)
        else:
            self.stream.write('Tests: ' + green("OK"))
        if skipped:
            infos.append("skipped=%d" % skipped)
        if expectedFails:
            infos.append("expected failures=%d" % expectedFails)
        if unexpectedSuccesses:
            infos.append("unexpected successes=%d" % unexpectedSuccesses)
        if infos:
            self.stream.write(" (%s)" % (", ".join(infos),))



'''
Created on Aug 5, 2013

@author: sean
'''
from __future__ import print_function
import unittest
from os.path import dirname,join
from argparse import ArgumentParser
from binstar_client.tests.runner import ColorTextTestRunner

def main():
    
    parser = ArgumentParser()
    parser.add_argument('--html', action='store_true')
    parser.add_argument('source_dir', nargs='?', default='')
    args = parser.parse_args()
    import binstar_client
    print(binstar_client)
    loader = unittest.loader.TestLoader()
    discover_dir = join(dirname(binstar_client.__path__[0]), args.source_dir)
    print('Discover %s' % discover_dir)
    tests = loader.discover(discover_dir)
    runner = ColorTextTestRunner(verbosity=2)
    result = runner.run(tests)
    runner.write_end(result)

    exit(0 if result.wasSuccessful() else -1)
    
    
if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""Test anaconda-client configuration set/get."""

import os
import shutil
import tempfile
import unittest
# Standard library imports
from os.path import join

# Third party imports
import mock
import yaml

# Local imports
from binstar_client.utils import config
from binstar_client.errors import BinstarError


class Test(unittest.TestCase):
    CONFIG_DATA = {'ssl_verify': False}

    def create_config_dirs(self):
        tmpdir = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, tmpdir)
        system_dir = join(tmpdir, 'system')
        user_dir = join(tmpdir, 'user')
        os.mkdir(system_dir)
        os.mkdir(user_dir)
        return user_dir, system_dir

    def test_defaults(self):
        user_dir, system_dir = self.create_config_dirs()
        with open(join(user_dir, 'config.yaml'), 'wb') as fd:
            fd.write(b'')

        with mock.patch('binstar_client.utils.config.SEARCH_PATH',
                        [system_dir, user_dir]):
            cfg = config.get_config()
            self.assertEqual(cfg, config.DEFAULT_CONFIG)

    def test_global_url(self):
        """
        Test regression reported on:

        https://github.com/Anaconda-Platform/anaconda-client/issues/464
        """
        user_dir, system_dir = self.create_config_dirs()
        with open(join(user_dir, 'config.yaml'), 'wb') as fd:
            fd.write(b'')

        url_data = {'url': 'https://blob.org'}
        config_data = config.DEFAULT_CONFIG.copy()
        config_data.update(url_data)

        with mock.patch('binstar_client.utils.config.SEARCH_PATH',
                        [system_dir, user_dir]):
            config.save_config(url_data, join(user_dir, 'config.yaml'))
            cfg = config.get_config()
            self.assertEqual(cfg, config_data)

    def test_merge(self):
        user_dir, system_dir = self.create_config_dirs()
        with open(join(system_dir, 'config.yaml'), 'wb') as fd:
            fd.write(b'''
ssl_verify: false
sites:
    develop:
        url: http://develop.anaconda.org
            ''')

        with open(join(user_dir, 'config.yaml'), 'wb') as fd:
            fd.write(b'''
ssl_verify: true
sites:
    develop:
        ssl_verify: false
            ''')

        with mock.patch('binstar_client.utils.config.SEARCH_PATH', [system_dir, user_dir]), \
                mock.patch('binstar_client.utils.config.DEFAULT_CONFIG', {}):
            cfg = config.get_config()

            self.assertEqual(cfg, {
                'ssl_verify': True,
                'sites': {
                    'develop': {
                        'url': 'http://develop.anaconda.org',
                        'ssl_verify': False,
                    }
                }
            })

    def test_support_tags(self):
        user_dir, system_dir = self.create_config_dirs()
        with open(join(user_dir, 'config.yaml'), 'wb') as fd:
            fd.write(b'''
!!python/unicode 'sites':
   !!python/unicode 'alpha': {!!python/unicode 'url': !!python/unicode 'foobar'}
   !!python/unicode 'binstar': {!!python/unicode 'url': !!python/unicode 'barfoo'}
ssl_verify: False
''')

        with mock.patch('binstar_client.utils.config.SEARCH_PATH', [user_dir]), \
                mock.patch('binstar_client.utils.config.DEFAULT_CONFIG', {}):
            cfg = config.get_config()

            self.assertEqual(cfg, {
                'ssl_verify': False,
                'sites': {
                    'alpha': {
                        'url': 'foobar',
                    },
                    'binstar': {
                        'url': 'barfoo',
                    },
                }
            })

    @mock.patch('binstar_client.utils.config.warnings')
    @mock.patch('binstar_client.utils.config.yaml_load', wraps=config.yaml_load)
    def test_load_config(self, mock_yaml_load, mock_warnings):
        tmpdir = tempfile.mkdtemp()
        tmp_config = os.path.join(tmpdir, 'config.yaml')

        with open(tmp_config, 'w') as f:
            config.yaml_dump(self.CONFIG_DATA, f)

        with self.subTest('OK'):
            self.assertEqual(self.CONFIG_DATA, config.load_config(tmp_config))
            mock_warnings.warn.assert_not_called()

        with self.subTest('yaml.YAMLError'):
            mock_yaml_load.side_effect = yaml.YAMLError
            self.assertEqual({}, config.load_config(tmp_config))
            self.assertTrue(os.path.exists(tmp_config + '.bak'))
            os.remove(tmp_config + '.bak')
            mock_warnings.warn.assert_called()

        mock_warnings.reset_mock()
        with self.subTest('PermissionError'):
            mock_yaml_load.side_effect = PermissionError

            self.assertEqual({}, config.load_config(tmp_config))
            mock_warnings.warn.assert_called()

        mock_warnings.reset_mock()
        with self.subTest('OSError'):
            mock_yaml_load.side_effect = OSError

            self.assertEqual({}, config.load_config(tmp_config))
            mock_warnings.warn.assert_not_called()

        shutil.rmtree(tmpdir)

    @mock.patch('binstar_client.utils.config.os.makedirs', wraps=os.makedirs)
    @mock.patch('binstar_client.utils.config.os.replace', wraps=os.replace)
    def test_save_config(self, mock_os_replace, mock_os_makedirs):
        config_filename = 'config.yaml'

        with self.subTest('OK'), tempfile.TemporaryDirectory() as test_config_dir:
            config_path = os.path.join(test_config_dir, config_filename)
            config.save_config(self.CONFIG_DATA, config_path)

            self.assertEqual(self.CONFIG_DATA, config.load_config(config_path))
            mock_os_makedirs.assert_called_once_with(test_config_dir, exist_ok=True)
            mock_os_replace.assert_called_once_with(config_path + '~', config_path)

        mock_os_replace.reset_mock()
        mock_os_makedirs.reset_mock()

        with self.subTest('OSError'), tempfile.TemporaryDirectory() as test_config_dir:
            mock_os_replace.side_effect = OSError

            config_path = os.path.join(test_config_dir, config_filename)
            with self.assertRaises(BinstarError):
                config.save_config(self.CONFIG_DATA, config_path)

            self.assertFalse(os.path.exists(config_path))
            mock_os_makedirs.assert_called_once_with(test_config_dir, exist_ok=True)
            mock_os_replace.assert_called_once_with(config_path + '~', config_path)
